const { join } = require('path');
const argv = require('minimist')(process.argv);
const cwd = process.cwd();
const env = process.env.NODE_ENV = process.env.NODE_ENV || 'development';
const compact = require('lodash/compact');
const { EnvironmentPlugin, IgnorePlugin, NoErrorsPlugin } = require('webpack');
const ExtractTextPlugin = require('extract-text-webpack-plugin');
const CopyWebpackPlugin = require('copy-webpack-plugin');
const nib = require('nib');
const StatsWriterPlugin = require("webpack-stats-plugin").StatsWriterPlugin

const config = {
  base: {
    context: cwd,
    devtool: 'cheap-module-source-map',
    target: 'web',
    entry: {},
    stylus: {
      use: [
        nib(),
      ],
    },
    output: {
      path: join(cwd, 'build'),
      filename: '[name].js',
      libraryTarget: 'umd',
      publicPath: '/',
    },
    resolve: {
      // tells webpack where to load modules from
      modules: [
        // Note: this allows you to use require 'components/dashboard' or 'actions/billing' from any file
        //join(cwd, 'app'),
        join(cwd, 'src'),
        join(cwd, 'src/components'),
        join(cwd, 'node_modules'),
      ],
      extensions: [
        '',
        '.js',
        '.jsx',
        '.json',
        '.yml',
        '.styl',
        '.css',
      ],
    },
    module: {
      noParse:[
        join(cwd, 'project'),
      ],
      /*
      preLoaders: [
        {
          test: /\.(js|jsx)$/,
          loader: 'eslint-loader',
          include: [
            join(process.cwd(),'src')
          ],
          exclude: [
            /node_modules/,
          ],
        }
      ],
      */
      loaders: [
        {
          test: /\.(js|jsx)$/,
          loader: 'babel',
          profile: true,
          query: {
            presets: ['benzinga-webpack', 'react-hmre'],
            only: ['src','app'],
            cacheDirectory: true,
          },
          exclude: [
            /node_modules/,
          ],
        },
        {
          test: /\.styl$/,
          loader: ExtractTextPlugin.extract('css-loader!stylus-loader'),
        },
        {
          test: /\.json$/,
          loader: 'json',
          ignore: [
            join(cwd,'project'),
            /node_modules/,
          ],
        },
        { test: /\.woff(\?v=.+)?$/, loader: 'file' },
        { test: /\.woff2(\?v=.+)?$/, loader: 'file' },
        { test: /\.ttf(\?v=.+)?$/, loader: 'file' },
        { test: /\.svg(\?v=.+)?$/, loader: 'file' },
        { test: /\.eot(\?v=.+)?$/, loader: 'file' },
        { test: /\.wav$/, loader: 'file' },
        { test: /\.ogg(\?v=.+)?$/, loader: 'file' },
        { test: /\.mp3(\?v=.+)?$/, loader: 'file' },
        { test: /\.png$/, loader: 'file' },
        { test: /\.jpg(\?v=\d+\.\d+\.\d+)?$/, loader: 'file' },
        { test: /\.gif(\?v=\d+\.\d+\.\d+)?$/, loader: 'file' },
        { test: /\.html(\?v=\d+\.\d+\.\d+)?$/, loader: 'file?name=[path][name].[ext]&context=src' },
      ],
    },
    plugins: [
      new NoErrorsPlugin(),
      new IgnorePlugin(/^\.\/locale$/, [/moment$/], join(cwd, 'project')),
      new ExtractTextPlugin('style.css'),
      new CopyWebpackPlugin([
        {from: 'static'},
      ]),
      new EnvironmentPlugin([
        'NODE_ENV',
        'BABEL_ENV',
        'SERVICES_ROOT',
        'API_ROOT',
        'MIXPANEL_KEY',
        'QUOTE_ADDR',
        'CONTENT_ADDR',
        'SEGMENT_KEY',
        'PRO_API',
        'OLARK_SITE_ID',
        'STRIPE_PUBLISHABLE_KEY',
      ]),
    ],
  },
};

function loadWindowGlobals() {
  const jsdom = require('jsdom');
  const context = {};

  context.document = jsdom.jsdom('<html />');
  context.document.defaultView.localStorage = {};

  require('lodash/defaults')(context, context.document.defaultView);

  context.window = context.document.defaultView;
  context.window.serverSide = true;

  return context;
}

module.exports = (stageName = 'development') => {
  const stage = require(`./webpack.${stageName}.babel`)({...config.base})

  const staticLocals = stageName === 'dll'
    ? {}
    : loadWindowGlobals()

  if (stageName === 'development' || stageName === 'production' || stageName === 'staging') {
    stage.plugins.push(
      new StatsWriterPlugin({
        fields: null,
        filename: `project/stats/${stageName}.stats.json`,
      })
    )
  }

  stage.plugins = compact(stage.plugins);
  stage.module.loaders = compact(stage.module.loaders);

  stage.staticLocals = staticLocals || {};

  return stage;
};
