jest.dontMock('../CCNumberInput');
jest.dontMock('../../utils');
jest.dontMock('../../constants');

import React from 'react';
import ReactDOM from 'react-dom';
import TestUtils from 'react-addons-test-utils';

const CCNumberInput = require('../CCNumberInput').default;
const callback = jest.genMockFn();

describe('CCNumberInput lifecycle tests', () => {
  let component;
  let input;

  beforeEach(() => {
    component = TestUtils.renderIntoDocument(<CCNumberInput onStateUpdate={callback}/>);
    input = component.refs.ccNumberInput;
  });

  it('renders properly on load', () => {
    // Should exist
    expect(input).toBeTruthy();
    // Should be empty
    expect(input.textContent).toBe('');
    // Should have an empty state
    expect(component.state.value).toBe('');
  });

  it('handles onKeyPress properly', () => {
    // Set up some mocks to make sure we're calling the methods
    component.restrictCardNumber = jest.genMockFn();
    component.formatCardNumber = jest.genMockFn();

    // Trigger
    TestUtils.Simulate.keyPress(input, {keyCode: 65});

    // Check it
    expect(component.restrictCardNumber).toBeCalled();
    expect(component.formatCardNumber).toBeCalled();
  });

  it('handles onKeyDown properly', () => {
    // Setup some mocks to test
    component.formatBackCardNumber = jest.genMockFn();

    // Trigger
    TestUtils.Simulate.keyDown(input, {keyCode: 65});

    // Check it
    expect(component.formatBackCardNumber).toBeCalled();
  });

  it('handles onKeyUp properly', () => {
    // Setup some mocks to test
    component.setCardType = jest.genMockFn();

    // Trigger
    TestUtils.Simulate.keyUp(input, {keyCode: 65});

    // Check it
    expect(component.setCardType).toBeCalled();
  });

  it('handles onPaste properly', () => {
    // Setup some mocks to test
    component.reformatNumeric = jest.genMockFn();
    component.reformatCardNumber = jest.genMockFn();

    input.value = '4111111111111111';

    // Trigger
    TestUtils.Simulate.paste(input);

    // Check it
    expect(component.reformatNumeric).toBeCalled();
    expect(component.reformatCardNumber).toBeCalled();
  });

  it('handles onChange properly', () => {
    // Setup some mocks to test
    component.reformatCardNumber = jest.genMockFn();

    input.value = '4111';

    // Trigger
    TestUtils.Simulate.change(input);

    // Check it
    expect(component.reformatCardNumber).toBeCalled();
  });

  it('handles onInput properly', () => {
    // Setup some mocks to test
    component.reformatCardNumber = jest.genMockFn();
    component.setCardType = jest.genMockFn();

    input.value = '4111111111111111';

    // Trigger
    TestUtils.Simulate.input(input);

    // Check it
    expect(component.reformatCardNumber).toBeCalled();
    expect(component.setCardType).toBeCalled();
  });

  it('ensures card state is updated on component and parent', () => {
    // Card number
    component.updateState('4111 1111 1111 1111');
    expect(component.state.value).toBe('4111 1111 1111 1111');
    expect(callback).toBeCalled();
    expect(callback).toBeCalledWith({card: '4111 1111 1111 1111'});
  });

  it('ensures card type state is updated on component and parent', () => {
    // Card number
    component.updateState('visa', 'cardType');
    expect(component.state.cardType).toBe('visa');
    expect(callback).toBeCalled();
    expect(callback).toBeCalledWith({cardType: 'visa'});
  });
});

describe('CCNumberInput formatter/validator tests', () => {
  let component;
  let input;

  beforeEach(() => {
    component = TestUtils.renderIntoDocument(<CCNumberInput onStateUpdate={callback}/>);
    input = component.refs.ccNumberInput;
  });

  it('ensures getCardType returns the right card', () => {
    expect(component.getCardType('4111111111111111')).toBe('visa');
    expect(component.getCardType('zac')).toBe('unknown');
  });

  it('ensures setCardType sets the right card type in state', () => {
    // Dummy event
    const event = {
      keyCode: 55,
      which: 55,
      target: {value: '4111111111111111'},
      preventDefault: jest.genMockFn()
    };

    // Set it
    component.setCardType(event);

    // Check it
    expect(component.state.cardType).toBe('visa');
  });

  it('ensures setCardType sets an unknown card type in state', () => {
    // Dummy event
    const event = {
      keyCode: 55,
      which: 55,
      target: {value: 'zac'},
      preventDefault: jest.genMockFn()
    };

    // Set it
    component.setCardType(event);

    // Check it
    expect(component.state.cardType).toBe('unknown');
  });

  it('ensures getFormattedCardNumber returns a formatted number', () => {
    expect(component.getFormattedCardNumber('4111111111111111'))
      .toBe('4111 1111 1111 1111');
    expect(component.getFormattedCardNumber('zachary')).toBe('');
    expect(component.getFormattedCardNumber('911')).toBe('911');
    expect(component.getFormattedCardNumber('375987654321001'))
      .toBe('3759 876543 21001');
  });

  it('ensures reformatNumeric strips non-digits', () => {
    // Dummy event
    const event = {target: { value: '123a4' } };

    // Set it
    component.reformatNumeric(event);

    // Check it
    expect(component.state.value).toBe('1234');
  });

  it('ensures formatBackCardNumber breaks when not backspacing', () => {
    // Dummy event
    const event = {
      keyCode: 55,
      which: 55,
      target: {value: 'zac'},
      preventDefault: jest.genMockFn()
    };

    const result = component.formatBackCardNumber(event);
    expect(result).toBeUndefined();
  });

  it('ensures formatBackCardNumber breaks when focus is off', () => {
    // Dummy backspace event
    const event = {
      keyCode: 8,
      which: 8,
      target: {
        value: '41111',
        selectionStart: 1
      },
      preventDefault: jest.genMockFn()
    };

    const result = component.formatBackCardNumber(event);
    expect(result).toBeUndefined();
  });

  it('ensures formatBackCardNumber removes the space/last digit', () => {
    // Dummy backspace event
    const event = {
      keyCode: 8,
      which: 8,
      target: {
        value: '4111 1',
        selectionStart: null
      },
      preventDefault: jest.genMockFn()
    };

    component.formatBackCardNumber(event);
    expect(component.state.value).toBe('4111 ');
  });

  it('ensures formatBackCardNumber removes the trailing space/last digit', () => {
    // Dummy backspace event
    const event = {
      keyCode: 8,
      which: 8,
      target: {
        value: '4111 ',
        selectionStart: null
      },
      preventDefault: jest.genMockFn()
    };

    component.formatBackCardNumber(event);
    expect(component.state.value).toBe('411');
  });

  it('ensures reformatCardNumber forces a reformat/state update', () => {
    // Dummy event
    const event = {
      keyCode: 55,
      which: 55,
      target: {value: '4111111111111111'},
      preventDefault: jest.genMockFn()
    };

    // Set it
    component.reformatCardNumber(event);

    // Check it
    expect(component.state.value).toBe('4111 1111 1111 1111');
    expect(callback).toBeCalled();
    expect(callback).toBeCalledWith({card: '4111 1111 1111 1111'});
  });

  it('ensures formatCardNumber breaks on non-digits', () => {
    // Dummy event
    const event = {
      keyCode: 65,
      which: 65,
      target: {value: '411'},
      preventDefault: jest.genMockFn()
    };

    const result = component.formatCardNumber(event);
    expect(result).toBeUndefined();
  });

  it('ensures formatCardNumber breaks on too many digits', () => {
    // Dummy event
    const event = {
      keyCode: 52,
      which: 52,
      target: {value: '4111111111111111'},
      preventDefault: jest.genMockFn()
    };

    const result = component.formatCardNumber(event);
    expect(result).toBeUndefined();
  });

  it('ensures formatCardNumber breaks if the focus is off', () => {
    // Dummy event
    const event = {
      keyCode: 49,
      which: 49,
      target: {value: '4111 1111 1111', selectionStart: 4},
      preventDefault: jest.genMockFn()
    };

    const result = component.formatCardNumber(event);
    expect(result).toBeUndefined();
  });

  it('ensures formatCardNumber handles Amex properly', () => {
    // Dummy event
    const event = {
      keyCode: 50,
      which: 50,
      target: {value: '3759 876543'},
      preventDefault: jest.genMockFn()
    };

    component.formatCardNumber(event);
    expect(component.state.value).toBe('3759 876543 2');
  });

  it('ensures formatCardNumber handles non-Amex properly', () => {
    // Dummy event
    const event = {
      keyCode: 49,
      which: 49,
      target: {value: '4111 1111'},
      preventDefault: jest.genMockFn()
    };

    component.formatCardNumber(event);
    expect(component.state.value).toBe('4111 1111 1');
  });

  it('ensures formatCardNumber updates non-spaced state', () => {
    // Dummy event
    const event = {
      keyCode: 49,
      which: 49,
      target: {value: '4111 111'},
      preventDefault: jest.genMockFn()
    };

    component.formatCardNumber(event);
    expect(component.state.value).toBe('4111 1111 ');
  });

  it('ensures restrictCardNumber doesnt allow letters', () => {
    // Dummy event
    const event = {
      keyCode: 65,
      which: 65,
      target: {value: ''},
      preventDefault: jest.genMockFn()
    };
    // We dont need to restrict this
    expect(component.restrictCardNumber(event)).toBeTruthy();
  });

  it('ensures restrictCardNumber handles focus properly', () => {
    // Dummy event
    const event = {
      keyCode: 53,
      which: 53,
      target: {value: '12345', selectionStart: 3, selectionEnd: 4},
      preventDefault: jest.genMockFn()
    };

    expect(component.restrictCardNumber(event)).toBeTruthy();
  });

  it('ensures restrictCardNumber limits the amount of digits', () => {
    // Dummy event
    const event = {
      keyCode: 49,
      which: 49,
      target: {value: '4111 1111 1111 1111'},
      preventDefault: jest.genMockFn()
    };

    expect(component.restrictCardNumber(event)).toBeFalsy();
  });

  it('ensures restrictCardNumber limits the amount of digits', () => {
    // Dummy event
    const event = {
      keyCode: 49,
      which: 49,
      target: {value: '12345123451234512345'},
      preventDefault: jest.genMockFn()
    };

    expect(component.restrictCardNumber(event)).toBeFalsy();
  });
});
